/* Object class for a point spread function. Sparse PSFs are efficiently
 * stored. Automatically centers the PSF, in a weighted sense, to the nearest
 * pixel at least.
 */
#ifndef PSF_H
#define PSF_H

#ifdef __cplusplus
extern "C" {
#endif

#include "byte_image_file.h"
#include "parallel.h"

typedef struct {
  long i;
  long j;
  double w;
} PsfEntry;

typedef struct {
  ulong numEntries;
  PsfEntry *_entries;
} Psf;

/* Constructor from a grayscale DoubleImage.
 */
void psf_new(Psf **p_this, DoubleImage *image);

/* Constructor from a grayscale image file.
 */
void psf_new_from_filename(Psf **p_this, char *filename);

/* Destructor.
 */
void psf_delete(Psf **p_this);

/* Print.
 */
void psf_print(Psf *this, Printer *p);

/* Blur a DoubleImage with a Psf.
 */
DoubleImage *double_image_new_blur_psf(
    DoubleImage **p_this,
    DoubleImage *other,
    Psf *psf);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* _H */
