/* Static library providing the Unblur algorithm for deblurring an image in
 * position space. See johncostella.com/unblur for more details.
 */
#ifndef UNBLUR_H
#define UNBLUR_H

#ifdef __cplusplus
extern "C" {
#endif

#include "Psf.h"
#include "parallel.h"
#include "qsortx.h"

/* Deblur an image by applying the UnBlur algorithm to it.
 */
DoubleImage *double_image_new_unblur(
    DoubleImage **p_this,
    DoubleImage *other,
    Psf *psf,
    DoubleImage *approx,
    double tolerance,
    ulong max_iterations,
    Logger *l);

/* Create a new DoubleImage by back-propagating with a PSF.
 */
DoubleImage *double_image_new_back_propagate_psf(
    DoubleImage **p_this,
    DoubleImage *other,
    Psf *psf,
    Logger *l);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* _H */
