#include "make_linear_blur.h"

int main(int argc, char *argv[]) {
  bool slash;
  byte quality;
  char *filename;
  ulong blur_height;
  ulong blur_width;
  ulong i;
  ulong image_height;
  ulong image_width;
  ulong j;
  ByteImagePixel p;
  ByteImage *_out = null;

  args_description("Make a linear blur point spread function image.");
  args_string("filename", "Filename of image file to be created");
  args_integer("width", "Width of the linear blur");
  args_integer("height", "Height of the linear blur");
  args_flag("slash", 's', "Blur looks like a slash");
  args_flag("backslash", 'b', "Blur looks like a backslash");
  args_flag("png", 'p', "Save the output file in PNG format");
  args_flag("jpeg", 'j', "Save the output file in JPEG format");
  args_optional_integer(
      "jpeg-quality",
      'q',
      jpeg_file_get_default_quality(),
      "The JPEG quality to use for the output JPEG file, between 0 and 100");
  args_parse(argc, argv);

  blur_width = args_get_positive_integer("width");
  blur_height = args_get_positive_integer("height");

  if (blur_width > 1 && blur_height > 1) {
    if (args_set_veto("slash", "backslash")) {
      slash = true;
    } else if (args_set("backslash")) {
      slash = false;
    } else {
      throw("Either --slash or --backslash needs to be specified when blur "
          "is not horizontal or vertical");
    }
  } else if (blur_width == 1 && blur_height == 1) {
    throw("Cannot have a one-pixel blur");
  }

  image_width = blur_width + 2;
  image_height = blur_height + 2;

  byte_image_new(&_out, image_width, image_height);

  p.a = p.r = p.g = p.b = 255;

  if (blur_width == 1) {
    for (j = 0; j < blur_height; j++) {
      byte_image_set(_out, 1, j + 1, p);
    }
  } else if (blur_height == 1) {
    for (i = 0; i < blur_width; i++) {
      byte_image_set(_out, i + 1, 1, p);
    }
  } else if (slash) {
    if (blur_width > blur_height) {
      for (i = 0; i < blur_width; i++) {
        j = blur_height - 1
          - (i * (blur_height - 1) + (blur_width - 1) / 2) / (blur_width - 1);
        byte_image_set(_out, i + 1, j + 1, p);
      }
    } else {
      for (j = 0; j < blur_height; j++) {
        i = blur_width - 1
          - (j * (blur_width - 1) + (blur_height - 1) / 2) / (blur_height - 1);
        byte_image_set(_out, i + 1, j + 1, p);
      }
    }
  } else {
    if (blur_width > blur_height) {
      for (i = 0; i < blur_width; i++) {
        j = (i * (blur_height - 1) + (blur_width - 1) / 2) / (blur_width - 1);
        byte_image_set(_out, i + 1, j + 1, p);
      }
    } else {
      for (j = 0; j < blur_height; j++) {
        i = (j * (blur_width - 1) + (blur_height - 1) / 2) / (blur_height - 1);
        byte_image_set(_out, i + 1, j + 1, p);
      }
    }
  }

  filename = args_get_string("filename");
  quality = args_get_bounded_integer("jpeg-quality", 0, 100);

  if (args_get_flag_veto("png", "jpeg")) {
    byte_image_save_png(_out, filename);
  } else if (args_get_flag("jpeg")) {
    byte_image_save_jpeg(_out, filename, quality);
  } else {
    byte_image_save_quality(_out, filename, quality);
  }

  byte_image_delete(&_out);

  return 0;
}
