#include "make_magic_blur.h"
#include <math.h>

int main(int argc, char *argv[]) {
  byte quality;
  char *filename;
  ulong center;
  ulong height;
  ulong width;
  double blur;
  DoubleImage *_din = null;
  DoubleImage *_dout = null;
  ByteImage *_out = null;

  args_description("Make a magic blur point spread function image.");
  args_string("filename", "Filename of image file to be created");
  args_real("blur", "The amount of blur");
  args_flag("png", 'p', "Save the output file in PNG format");
  args_flag("jpeg", 'j', "Save the output file in JPEG format");
  args_optional_integer(
      "jpeg-quality",
      'q',
      jpeg_file_get_default_quality(),
      "The JPEG quality to use for the output JPEG file, between 0 and 100");
  args_parse(argc, argv);

  blur = args_get_positive_real("blur");

  width = height = 3 * (ulong)ceil(blur) / 2 * 2 + 1;
  center = (width - 1) / 2;

  double_image_new_y(&_din, width, height);
  _din->_channels[0][center][center] = 1;

  double_image_new_blur(
      &_dout,
      _din,
      blur);
  double_image_delete(&_din);
  double_image_auto_scale(_dout);

  byte_image_new_from_double(&_out, _dout);
  double_image_delete(&_dout);

  filename = args_get_string("filename");
  quality = args_get_bounded_integer("jpeg-quality", 0, 100);

  if (args_get_flag_veto("png", "jpeg")) {
    byte_image_save_png(_out, filename);
  } else if (args_get_flag("jpeg")) {
    byte_image_save_jpeg(_out, filename, quality);
  } else {
    byte_image_save_quality(_out, filename, quality);
  }

  byte_image_delete(&_out);

  return 0;
}
